#! perl -w

# Licensed Materials - Property of HCL and/or IBM
# Copyright HCL Technologies Ltd. 2016, 2020. All Rights Reserved.
# Copyright IBM Corporation 1999, 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights -  Use, duplication or 
# disclosure restricted by GSA ADP Schedule.



# Win9x command.com does not interact with perl according to win32 standard.
# We therefore need to call some functions in a non-standard way.
#
# This detects if the system is Win95 for that purpose.

# Miscellaneous functions
use File::Copy;
use File::Spec;

my $VERBOSE_MODE=$ENV{ATTOLSTUDIO_VERBOSE};

my $RTIST_TARGET_PROJECT;
my $RTIST_TOP_CAPSULE_BASENAME;
my $RTIST_OTE_PROJECT = '';
my $targetfilename;
our $ATC_INFORMATION;
$TARGETDIR = '';
$SCRIPTDIR = '';

sub mySystem {
    my( $OS, $ROSERT_HOST );

    $OS          = '?' unless $OS          = $ENV{'OS'};
    $ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};
    if ($VERBOSE_MODE) {
        print STDERR "====================================\n";
        print STDERR "system call @_\n";
        print STDERR "====================================\n";
    }
    if (($ROSERT_HOST eq 'win32') && ($OS !~ m{^Windows_NT$}i )) {
        return system( join( ' ', @_ ) );
    } else {
        return system( @_ );
    }
}

sub findAppTcjs {
    my $mkfile = "0.mk";
    my $appfilename = "";
    open(my $fh, '<:encoding(UTF-8)', $mkfile) or die "Could not open file '$mkfile' $!";
 
    while (my $line = <$fh>) {
        chomp $line;
        if ($line =~ m/^# Generated from (.*)/) {
            $appfilename = $1;
            last;
        }
    }
    close ($fh);
    
    if ($VERBOSE_MODE) {
        print STDERR "============== finding app.tcjs ==================\n";   
        print STDERR "appfilename = ".$appfilename."\n";
        print STDERR "====================================\n";   
    }
    return $appfilename;
}

sub findRTS_HOME {
    my $mkfile = "makefile";
    my $RTS_HOME = "";
    open(my $fh, '<:encoding(UTF-8)', $mkfile) or die "Could not open file '$mkfile' $!";
    while (my $line = <$fh>) {
        chomp $line;
        if ($line =~ m/^RTS_HOME = (.*)$/) {
            $RTS_HOME = $1;
            last;
        }
    }
    close ($fh);
    if ($VERBOSE_MODE) {
        print STDERR "============== finding RTS_HOME ==================\n";   
        print STDERR "RTS_HOME = ".$RTS_HOME."\n";
        print STDERR "====================================\n";   
    }
    return $RTS_HOME;    
}


sub analyseTcjsFile {
    my $RTIST_APP_TCJS = shift;
    if ($RTIST_APP_TCJS eq '') {
        return;
    }
    if (-e $RTIST_APP_TCJS) {
        open(my $fh, '<:encoding(UTF-8)', $RTIST_APP_TCJS) or die "Could not open file '$RTIST_APP_TCJS' $!";
        while (my $line = <$fh>) {
            chomp $line;
            if ($line =~ m/^tc.targetProject = '\/(.*)';/) {
                $RTIST_TARGET_PROJECT = $1;
            }
            if ($line =~ m/^tc.instrumentationData = { project: '(.*)'(.*)};/) {
                $RTIST_OTE_PROJECT = $1;
            }
            if ($line =~ m/^tc.topCapsule = 'platform:\/resource\/(.*)\/(\w*).e[fm]x#/) {
                $RTIST_TOP_CAPSULE_BASENAME = $2;
            }                                
        }
        close ($fh);
    } else {
        print STDERR "app.tcjs not found : ".$RTIST_APP_TCJS."\n";
    }
}

sub generateLocalEnvFileAndClean {
    # generate a local "envTestRTcc" that reference global one and override some stuff
    # do not generate if it already exists
    my ($filename) = @_;
 
    if (-e $filename) {
        #already generated
    } else { 
        # clean up
 
        if ($VERBOSE_MODE) {
            print STDERR "============== generating ".$filename."======================\n";
            print STDERR "location : \n";
            system("pwd");
            print STDERR "====================================\n";        
        } 
        # find and analyze tcjs file
        $RTIST_APP_TCJS = findAppTcjs();
        analyseTcjsFile($RTIST_APP_TCJS);
        
        # overwite tcjs values with parameters
        # possible args :
        # -TARGETPROJECT='<value>'
        #       ex : -TARGETPROJECT='/TrafficLight target with space'
        #       (starts with a '/' as in tcjs file)
        # -OTEPROJECT='<value>'
        #       ex : -OTEPROJECT='my OTE with spaces'
        #       (does not starts with a '/' as in tcjs file instrumentationData)
        # -TOPCAPSULEBASENAME='<value>'
        #       ex : -TOPCAPSULEBASENAME='TrafficLightComponent'
        my $targetOTEProjectIsFullPath=0;
        foreach $arg ( @ARGV ) {              
            if ($arg =~/^"?-TARGETPROJECT='?\/?([^'"]*)\['"]*/) {
                $RTIST_TARGET_PROJECT = $1;
                if ($arg =~/^"/) {
                    # path is directly a full path, and so, do not use map to retrieve it
                    $targetOTEProjectIsFullPath=1;
                }
                if ($VERBOSE_MODE) {
                    print STDERR "overwritten : target project to ".$RTIST_TARGET_PROJECT."\n";
                }
            }
            if ($arg =~/^-OTEPROJECT='?(.*)'?/) {
                $RTIST_OTE_PROJECT = $1;
                if ($VERBOSE_MODE) {
                    print STDERR "overwritten : ote project to ".$RTIST_OTE_PROJECT."\n";
                }
            }
            if ($arg =~/^-TOPCAPSULEBASENAME=?'(.*)?'/) {
                $RTIST_TOP_CAPSULE_BASENAME = $1;
                if ($VERBOSE_MODE) {
                    print STDERR "overwritten : topcapsule basename to ".$RTIST_TOP_CAPSULE_BASENAME."\n";
                }
            }
        }
        
        
        $RTIST_APP_TCJS =~ s/\\/\//g;
        my $RTS_HOME = findRTS_HOME();
        $RTS_HOME =~ s/\\/\//g;
        
        # load general environment file to retrieve eclipse project map
        $targetfilename = $ENV{'USERPROFILE'}."/envTestRTcc.pl";
        $targetfilename=~ s/\\/\//g;
        if (-e $targetfilename) {
            require $targetfilename;
        } else {
            print $targetfilename." not found: create a OneTestEmbedded project and save settings before compiling...\n";
            die;
        }
        require $SCRIPTDIR . "Fcts.pl";
        if (defined $RTIST_OTE_PROJECT) {
            if ($VERBOSE_MODE) {
                print STDERR "============== target OTE project '".$RTIST_OTE_PROJECT."' ======================\n";
                
                while( my ($k,$v) = each(%ECLIPSE_PROJECT_MAP) ) {
                    print STDERR "Clef=$k Valeur=$v\n";
                }
            }        
            if (exists( $ECLIPSE_PROJECT_MAP{$RTIST_OTE_PROJECT} )) {
                $targetfilename = $ECLIPSE_PROJECT_MAP{$RTIST_OTE_PROJECT}."/build/envTestRTcc.pl";
                $targetfilename=~ s/\\/\//g;
                if ($VERBOSE_MODE) {
                    print STDERR "============== found target OTE project ".$targetfilename." ======================\n";
                }
            }
        }
        copy($targetfilename, ".");
        # ok, we have good one, let's reload it
        if (-e $targetfilename) {
            require $targetfilename;
        } else {
            print $targetfilename." not found: create a OneTestEmbedded project and save settings before compiling...\n";
            die;
        }        
        my $project_location=$WORKSPACE_PATH."/".$PROJECT_NAME;
        
        if (exists( $ECLIPSE_PROJECT_MAP{$PROJECT_NAME} )) {
            $project_location=$ECLIPSE_PROJECT_MAP{$PROJECT_NAME};
        }
        if ($targetOTEProjectIsFullPath == 0) {
            if (exists( $ECLIPSE_PROJECT_MAP{$RTIST_TARGET_PROJECT} )) {
                $RTIST_TARGET_PROJECT=$ECLIPSE_PROJECT_MAP{$RTIST_TARGET_PROJECT};
            } else {
                $RTIST_TARGET_PROJECT=$WORKSPACE_PATH."\/".$RTIST_TARGET_PROJECT;
            }
        }
        # remove old logfile and out_file : if we recompile, it's obsolete
        $LOGFILENAME=$project_location."/".$LOGFILENAME;
        $ATL_OUT_FILE=$project_location."/".$ATL_OUT_FILE;
        $ATL_OUT_FILE=~ s/\\/\//g;
        if (-e $LOGFILENAME) {
            unlink($LOGFILENAME) or die "Can't delete $LOGFILENAME: $!\n";
        }
        if (-e $ATL_OUT_FILE) {
            unlink($ATL_OUT_FILE) or die "Can't delete $ATL_OUT_FILE: $!\n";
        }
        
       
        # used to store modified fdc
        my $additional_fdc_folder = "opt_fdc"; 
        # used to store original fdcs when remplaced by modified ones
        my $temporary_fdc_folder = "tmp_fdc"; 
        
        
        my $pwd=GetCurrentDir();
        $pwd=~ s/\\/\//g;
        my $localCopyEnvFile = File::Spec->rel2abs("envTestRTcc.pl");
        $localCopyEnvFile=~ s/\\/\//g;
        # generation
        open(FH, '>', $filename) or die $!;
        
        print FH "##!perl -w\n";
        print FH "# warning: generated file, all edition may be erased!!! \n";
        print FH "require \"".$localCopyEnvFile."\";\n";
        print FH "# local overloading \n";
        print FH "\$INTDIR='';\n";
        print FH "\$OUTDIR='.';\n";        
        print FH "\$LOGFILENAME=\"".$LOGFILENAME."\";\n";
        print FH "\$ATL_OUT_FILE=\"".$ATL_OUT_FILE."\";\n";
        print FH "\$MODEL_POST_SCRIPT=\"rtistCovColorModel.pl\";\n";
        print FH "\$XTPFILE=\$LOGFILENAME;\n";
        print FH "\$RTIST_APP_TCJS_FILE=\"".$RTIST_APP_TCJS."\";\n";        
        print FH "\$RTIST_TARGET_BUILD=\"".$pwd."\";\n"; 
        print FH "\$RTIST_ADDITIONAL_FDC=\$RTIST_TARGET_BUILD.\"/".$additional_fdc_folder."\";\n";
        print FH "\$RTIST_TEMPORARY_FDC=\$RTIST_TARGET_BUILD.\"/".$temporary_fdc_folder."\";\n";
        print FH "\$RTIST_TARGET_PROJECT=\"".$RTIST_TARGET_PROJECT."\";\n";
        print FH "\$RTIST_TARGET_COLORIZATION=\$RTIST_TARGET_PROJECT.\"/".$RTIST_TOP_CAPSULE_BASENAME.".mcov\";\n";
        print FH "push \@ATG_EXCLUDED_DIRS , \"$RTS_HOME\";\n";
        print FH "\$ENV{'ATLTGT'} = \$TARGETDIR;\n";
        
        print FH "1;\n";
        
        close(FH);
        # check if hitcount is plugged or not
        unless ($ATC_INFORMATION == 2) {
            # hit count is not present
            print "Warning : Hit count for model colorization is not active\n";
            print "\tto active it, open OTE build settings, Code coverage => Instrumentation control\n";
            print "\tand change Information mode to \"report hit count\"\n";
        }
    }
}

my $status = 0;
my @command;
my $localTestrtEnvFile = 'envTestRTRtistcc.pl';





generateLocalEnvFileAndClean($localTestrtEnvFile);

require $localTestrtEnvFile;

push @command, "trtcc";
push @command, "-config";
push @command, $localTestrtEnvFile;

if ($VERBOSE_MODE) {
    print STDERR "=============== CC raw =================\n";
    foreach $arg ( @ARGV ) {
        print STDERR $arg."\n";
    }
    print STDERR "====================================\n";
}

#if windows mode && visual tdp, adapt parameters

my @postcommand;
my @includes;
my @defines;
my @source;
my @targetobj;

my @filteredargs;

foreach $arg ( @ARGV ) {   
    if (($arg =~/^"?-TARGETPROJECT='?\/?(.*)'?"?/) ||
        ($arg =~/^-OTEPROJECT='?(.*)'?/) ||
        ($arg =~/^-TOPCAPSULEBASENAME='?(.*)'?/)) {
        # options are filtered before beeing passed to trccc
        if ($VERBOSE_MODE) {
            print STDERR "filtered : ".$arg."\n";
        }
    } else {
        push @filteredargs, $arg;
    }
}

if (($^O=~/win32/i) && ($TARGETDIR=~/visual/i)) {    
    while(@filteredargs) {
        $_=shift(@filteredargs);
        if (/^\/Fo(.*)$/) {
            # convert /Fo into -o    
            push @targetobj, "-o";
            push @targetobj, $1;
        } elsif(/^\"?\/I(.+)$/){   
            my $p=$1;$p=~s/\"$// if(s/^\"//);        
            push @includes, $p;
        } elsif(/^\"?\/D(.*)$/){
            my $p=$1;$p=~s/\"$// if(s/^\"//);        
            push @defines, $_;
        } elsif (/(.*)\.cpp\"?$/i) {
            push @source, $_;
        } else {
            push @postcommand, $_;
        }
    }
    push @command, @source;
    foreach $incl ( @includes ) {                  
        push @command, "-I".$incl."";
    }
    foreach $def ( @defines ) {                  
        push @command, "-D".$def."";
    }
    push @command, @targetobj;
    if (@postcommand != 0) {
        push @command, "--";
        push @command, "-CPPFLAGS-";
        push @command, @postcommand;
    }
    if ($VERBOSE_MODE) {
        print STDERR "=============== CC adapted (visual mode) =================\n";
        foreach $arg ( @command ) {
            print STDERR "\t".$arg."\n";
        }
        print STDERR "====================================\n";
    }
} else {
    foreach $arg ( @filteredargs ) {    
        push @command, $arg;
    }
}



if( $status == 0 ) {
    $status = &mySystem( @command );
}

exit 0;




